------------------------------------------------------------------------
-- AICombine:updateAIMovement
------------------------------------------------------------------------
function AutoTractor:updateAIMovement( superFunc, dt, ... )

	if self.acPause then return end
	
	if not self.isServer or self.acParameters == nil or not self.acParameters.enabled then
		return superFunc( self, dt, ... );
	end
	
	AutoTractor.statEvent( self, "t0", dt )

	AutoTractor.checkState( self )
	if not AutoSteeringEngine.hasTools( self ) then
		self:stopAITractor()
		return;
	end

	if not self.isControlled then
		if g_currentMission.environment.needsLights then
			self:setLightsVisibility(true)
		else
			self:setLightsVisibility(false)
		end
	end
	
	local allowedToDrive = AutoSteeringEngine.checkAllowedToDrive( self )
	
	if      allowedToDrive
			and ( self.acTurnStage  < 0
				 or self.acTurnStage == 3
				 or self.acTurnStage == 8
				 or self.acTurnStage == 17
				 or self.acTurnStage == 18
			   or self.acTurnStage == 27
				 or self.acTurnStage == 36
				 or self.acTurnStage == 38
			   or self.acTurnStage == 49
			   or self.acTurnStage == 57
			   or self.acTurnStage == 59 ) 
			and AutoSteeringEngine.checkIsAnimPlaying( self, self.acImplementsMoveDown ) then
		AutoTractor.setStatus( self, 3 )
		if    self.acAnimWaitTimer == nil then
			self.acAnimWaitTimer = self.acDeltaTimeoutWait
			allowedToDrive = false
		elseif self.acAnimWaitTimer > 0 then
			self.acAnimWaitTimer = self.acAnimWaitTimer - dt
			allowedToDrive = false
		end
	else
		self.acAnimWaitTimer = nil
	end
	
	--if not allowedToDrive then print("combine unloading") end
	
	for _, v in pairs(self.numCollidingVehicles) do
		if v > 0 then
			AutoTractor.setStatus( self, 3 )
			allowedToDrive = false
			--print("collision")
			break
		end
	end
	if self.waitForTurnTime > self.time then
		AutoTractor.setStatus( self, 0 )
		allowedToDrive = false
		--print("waiting: "..tostring( self.waitForTurnTime - self.time ))
	end
	local speedLevel = 1;
	if self.speed2Level ~= nil and 0 <= self.speed2Level and self.speed2Level <= 4 then
		speedLevel = self.speed2Level;
	end
	
-- Speed level always 1 while turning	
	if self.acTurnStage ~= 0 and 0 < speedLevel and speedLevel < 4 then
		speedLevel = 1
	end

	if not allowedToDrive or speedLevel == 0 then
		--print("not allowed to drive: "..tostring(allowedToDrive).." "..tostring(speedLevel));
		AutoTractor.statEvent( self, "tS", dt )
		AutoSteeringEngine.drive( self, dt, 0, 0, 0, 1, false, true, 0, 0.75 );
		return
	end
	
	if not self:getIsAITractorAllowed() then
		self:stopAITractor()
		return
	end

	local moveForwards = true

	local offsetOutside = 0;
	if     self.acParameters.rightAreaActive then
		offsetOutside = -1;
	elseif self.acParameters.leftAreaActive then
		offsetOutside = 1;
	end;
	
	self.turnTimer = self.turnTimer - dt;
	
	if     self.acTurnStage ~= 0 then
		self.aiRescueTimer = self.aiRescueTimer - dt;
	else
		self.aiRescueTimer = self.acDeltaTimeoutStop;
	end
	
	if self.aiRescueTimer < 0 then
		self:stopAITractor()
		return
	end
		
--==============================================================				
	local angle, angle2;
	local angleMax = self.acDimensions.maxLookingAngle;
	local detected = false;
	local fruitsDetected = false;
	local border   = 0;
	local angleFactor;
	local offsetOutside;
	local noReverseIndex = 0;
	
--==============================================================		
--==============================================================		
	local turnAngle = math.deg(AutoSteeringEngine.getTurnAngle(self));
	
	if AutoTractor.acDevFeatures then
		self.mogliInfoText = string.format( "Turn stage: %2i, angle: %3i",self.acTurnStage,turnAngle )
	end

	if self.acParameters.leftAreaActive then
		turnAngle = -turnAngle;
	end;

	fruitsDetected = AutoSteeringEngine.hasFruits( self, 0.9 )
	
	noReverseIndex = AutoSteeringEngine.getNoReverseIndex( self );

	--if     self.acTurnStage == -2
	--		or self.acTurnStage == -12
	--		or self.acTurnStage == -22
	--		or self.acTurnStage ==  27 then
	--	if self.acTurnStage ~= -2 then --and self.acParameters.headland > 0 then
	--		local _,z = AutoSteeringEngine.getTurnVector( self, true )
	--		if z > 0 then 
	--			fruitsDetected = false
	--		end
	--	end
	--	AutoSteeringEngine.enableCheckTurnPoint( self );
	--end

--==============================================================				
	if self.acTurnStage ~= 0 then
		self.acLast = nil
	end

	if self.acTurnStage <= 0 then
			
		if self.acTurnStage == 0 then
			local wx,_,wz = getWorldTranslation( self.acRefNode )
			local buffered = false
			local lsq = 99
			if      self.acLast ~= nil then
				lsq = Utils.vector2LengthSq( self.acLast.x - wx, self.acLast.z - wz )
				if      lsq < 0.01 then
					AutoTractor.statEvent( self, "t1", dt )
					buffered = true
				elseif  self.acLast.detected 					
						and self.acLast.angle2 + self.acLast.angle2 < angleMax 
						and lsq < 0.25 then
					AutoTractor.statEvent( self, "t2", dt )
					buffered = true
				elseif  self.acLast.detected 					
						and self.acLast.angle2 + self.acLast.angle2 < angleMax 
				    and lsq < 1 
				    and AutoSteeringEngine.getAllChainBorders( self ) < 1 then
					AutoTractor.statEvent( self, "t3", dt )
					buffered = true
				end
			end
			if buffered then
				AutoTractor.statEvent( self, "t4", dt )
				detected = self.acLast.detected
				angle2   = self.acLast.angle2
				border   = self.acLast.border
			else
				self.acLast = {}
				self.acLast.x = wx
				self.acLast.z = wz
				AutoSteeringEngine.currentSteeringAngle( self );
				if lsq >= 4 then
					AutoSteeringEngine.setChainOutside( self );
				else
					AutoSteeringEngine.setChainOutside( self, 1, 0.2 );
				end
				self.acLast.detected, self.acLast.angle2, self.acLast.border = AutoSteeringEngine.processChain( self );			
			end
			detected = self.acLast.detected
			angle2   = self.acLast.angle2
			border   = self.acLast.border
		else
			AutoSteeringEngine.currentSteeringAngle( self );
			AutoSteeringEngine.setChainOutside( self );
			detected, angle2, border = AutoSteeringEngine.processChain( self );			
		end
		
		self.acTurn2Outside = border > 0;
		
		if not detected then
			if self.acTurn2Outside and self.acTurnStage == 0 then
				angle = angleMax
			elseif self.acTurn2Outside then
				angle = self.acDimensions.maxSteeringAngle
			elseif self.acTurnStage == -3 or self.acTurnStage == -13 or self.acTurnStage == -23 then
				if self.acParameters.leftAreaActive then
					angle = -angle2;		
				else
					angle = -angle2;		
				end
				angle = math.max( 0, angle2 );
			elseif self.acTurnStage == -2 or self.acTurnStage == -12 or self.acTurnStage == -22 then
				angle  = nil
				angle2 = AutoSteeringEngine.navigateToSavePoint( self, true, maxLooking )

			elseif self.acTurnStage < 0 then --self.acTurnStage == -1 or self.acTurnStage == -11 or self.acTurnStage == -21 then
				angle = -self.acDimensions.maxSteeringAngle					
			else--if self.acParameters.upNDown or self.acParameters.frontPacker or  AutoSteeringEngine.noTurnAtEnd( self ) then
				angle = math.min( math.max( math.rad( turnAngle ), -self.acDimensions.maxSteeringAngle ), self.acDimensions.maxSteeringAngle )
			--else
			--	angle = -angleMax					
			end
		end
--==============================================================				
	elseif self.acTurnStage == 8 then	
		AutoSteeringEngine.currentSteeringAngle( self );
		AutoSteeringEngine.setChainStraight( self );			
		border = AutoSteeringEngine.getAllChainBorders( self );
		if border > 0 then detected = true end
	
--==============================================================				
	elseif self.acTurnStage == 27
			or self.acTurnStage == 36
			or self.acTurnStage == 38
			or self.acTurnStage == 49 then
		--AutoSteeringEngine.currentSteeringAngle( self );
		AutoSteeringEngine.setSteeringAngle( self, 0 )
		AutoSteeringEngine.setChainOutside( self );			
		detected, angle2, border = AutoSteeringEngine.processChain( self );					
		--border = AutoSteeringEngine.getAllChainBorders( self );
		--if border < 1 then detected = true end

--==============================================================				
	elseif self.acTurnStage == 59 then	
		AutoSteeringEngine.setSteeringAngle( self, 0 )
		AutoSteeringEngine.setChainStraight( self );			
		detected, angle2, border = AutoSteeringEngine.processChain( self );					
		
--==============================================================				
-- backwards
	elseif self.acTurnStage == 3 or self.acTurnStage == 18 then
		if self.acParameters.leftAreaActive == self.acTurn2Outside then
			AutoSteeringEngine.setSteeringAngle( self, -angleMax );
		else
			AutoSteeringEngine.setSteeringAngle( self, angleMax );
		end
		if self.acTurn2Outside then
			AutoSteeringEngine.setChainOutside( self );		
		else
			AutoSteeringEngine.setChainStraight( self );		
		end
		
		--if noReverseIndex > 0 then	
		--	if self.acTurn2Outside then
		--		border = 0;
		--	else
		--		border = 2;
		--	end
		--else
		border = AutoSteeringEngine.getAllChainBorders( self );
		--end
		
		if self.acTurn2Outside then
			angle    = -self.acDimensions.maxSteeringAngle;
		else
			angle    = self.acDimensions.maxSteeringAngle;
		end
		
		local testBorder = border < 1;
		
		if noReverseIndex > 0 then			
			detected = ( self.acTurn2Outside == testBorder )
			local toolAngle = AutoSteeringEngine.getToolAngle( self );			
			angle  = nil;
			angle2 = math.min( math.max( toolAngle, -self.acDimensions.maxSteeringAngle ), self.acDimensions.maxSteeringAngle );
		elseif self.acTurn2Outside == testBorder then
			angle = nil;
			detected, angle2, border = AutoSteeringEngine.processChain( self );					
			angle2 = -angle2
		end
	
			
--==============================================================		
-- U turn		
	elseif self.acTurnStage == 17 and turnAngle <= -105 then
		angle    = self.acDimensions.maxSteeringAngle;
	
		--if self.acParameters.leftAreaActive then
		--	self.acLastSteeringAngle = -angleMax;
		--else
		--	self.acLastSteeringAngle = angleMax;
		--end
		--AutoSteeringEngine.setSteeringAngle( self, 0 );
		--AutoSteeringEngine.setChainContinued( self );
		--AutoTractor.setChainOutside( self, angleFactor, -angleMax, angleMax );
		AutoSteeringEngine.setChainStraight( self );
		border = AutoSteeringEngine.getAllChainBorders( self );		
		detected = ( border <= 0 );
	else
		AutoSteeringEngine.setChainContinued( self );
	end

--==============================================================						
--==============================================================		
-- move far enough			
	if     self.acTurnStage == 1 then

		if turnAngle > -3 then
			angle = self.acDimensions.maxSteeringAngle;
		else
			angle = 0;
		end

		local x,z = AutoSteeringEngine.getTurnVector( self, false );
		if self.acTurn2Outside or z > self.acDimensions.radius then
			self.setAIImplementsMoveDown(self,false);
			self.acTurnStage   = self.acTurnStage + 1;
			self.turnTimer     = self.acDeltaTimeoutWait;
			allowedToDrive     = false;				
			self.waitForTurnTime = self.time + self.turnTimer;
		end

--==============================================================				
-- going back I
	elseif self.acTurnStage == 2 then
		
		moveForwards   = false;					
		local x,z = AutoSteeringEngine.getTurnVector( self, false );
		angle = -math.min( math.max( math.rad( turnAngle ), -self.acDimensions.maxSteeringAngle ), self.acDimensions.maxSteeringAngle )
		if z < self.acDimensions.radius + 1 then
			self.acTurnStage = self.acTurnStage + 1;
			if self.acTurn2Outside then
				self.turnTimer = self.acDeltaTimeoutWait;
			else
				self.turnTimer = 3 * self.acDeltaTimeoutWait;
			end
		end
		
--==============================================================				
-- going back II
	elseif self.acTurnStage == 3 then

		moveForwards = false;					

		if     detected
				or ( self.turnTimer < 0 
				 and ( self.acTurn2Outside or math.abs( turnAngle ) > 135 ) ) then
		--if     ( ( turnAngle > 30 or self.acTurn2Outside or noReverseIndex > 0  ) 
		--		 and detected 
		--		 and not fruitsDetected ) 
		--		or ( self.turnTimer < 0 and turnAngle > 90 ) then
			self.acTurnStage   = -1
			self.turnTimer     = self.acDeltaTimeoutWait;
			self.waitForTurnTime = self.time + self.turnTimer;
		end;

--==============================================================				
	elseif self.acTurnStage == 4 then
		self.acTurnStage   = -1

--==============================================================				
--==============================================================				
-- 90° corner w/o going reverse					
	elseif self.acTurnStage == 5 then
		allowedToDrive = false;				
		if self.acTurn2Outside then				
			angle = -self.acDimensions.maxSteeringAngle;
		else
			angle = self.acDimensions.maxSteeringAngle;
		end
		
		self.setAIImplementsMoveDown(self,false);
		self.acTurnStage   = 6;					
		
--==============================================================				
	elseif self.acTurnStage == 6 then
		if self.acTurn2Outside then				
			angle = -self.acDimensions.maxSteeringAngle;
		else
			angle = self.acDimensions.maxSteeringAngle;
		end
		
		if turnAngle < 0 then
			self.acTurnStage   = 7;					
		end;
		
--==============================================================				
	elseif self.acTurnStage == 7 then
		if self.acTurn2Outside then				
			angle = -self.acDimensions.maxSteeringAngle;
			if 170 < turnAngle and turnAngle < 180 then
				self.acTurnStage   = 8;					
			end;
		else
			angle = self.acDimensions.maxSteeringAngle;
			if 135 < turnAngle and turnAngle < 145 then
				self.acTurnStage   = 8;					
			end;
		end
		
--==============================================================				
		
	elseif self.acTurnStage == 8 then
		if self.acTurn2Outside then				
			angle = -self.acDimensions.maxSteeringAngle;
		else
			angle = self.acDimensions.maxSteeringAngle;
		end
		
		if detected or fruitsDetected then
			self.acTurnStage   = -1;					
			self.turnTimer     = self.acDeltaTimeoutStart;
		end;
		
--==============================================================				
--==============================================================				
-- wait before U-turn					
	elseif self.acTurnStage == 11 then
		allowedToDrive = false;						
		angle = 0;

		--if self.waitForTurnTime < self.time then
			self.setAIImplementsMoveDown(self,false);
			
			if self.acTurn2Outside then
				self.acTurnStage = 12;
				self.turnTimer   = self.acDeltaTimeoutStop;
			else
				local dist = self.acDimensions.uTurnDistance;
				if self.acParameters.noReverse then
					dist = self.acDimensions.uTurnDistance;
				end;

				if AutoSteeringEngine.getTurnDistance(self) > dist then
					if self.acParameters.leftAreaActive then
						AITractor.aiRotateLeft(self);
					else
						AITractor.aiRotateRight(self);
					end
					self.acTurnStage = 17;
					self.lastTurnAngle = math.deg(AutoSteeringEngine.getTurnAngle(self));					
				else
					self.acTurnStage = 15;
					self.turnTimer   = self.acDeltaTimeoutWait;
				end
			end
		--end
		
--==============================================================				
-- move to the right position before U-turn					
	elseif  self.acTurnStage == 12 then

		local ref = 0;
		if self.acParameters.noReverse then
			ref = math.deg(self.acDimensions.uTurnAngle);
		end;
		
		if self.acTurn2Outside then
			angle = -self.acDimensions.maxSteeringAngle;
									
			if turnAngle >= ref then
				self.acTurn2Outside = false;
				self.acTurnStage = 13;
				self.turnTimer   = self.acDeltaTimeoutRun;
			end
		else
			angle = self.acDimensions.maxSteeringAngle;
							
			if turnAngle <= 0 then
				local dist = self.acDimensions.uTurnDistance;

				if AutoSteeringEngine.getTurnDistance(self) > dist then
					if self.acParameters.leftAreaActive then
						AITractor.aiRotateLeft(self);
					else
						AITractor.aiRotateRight(self);
					end
					self.acTurnStage = 17;
					self.lastTurnAngle = math.deg(AutoSteeringEngine.getTurnAngle(self));					
				else
					self.acTurnStage = 14;
					self.turnTimer   = self.acDeltaTimeoutRun;
				end;
			end;
		end;

--==============================================================				
-- wait during U-turn
	elseif self.acTurnStage == 13 then
		allowedToDrive = false;						
		
		angle = self.acDimensions.maxSteeringAngle;
		
		if self.turnTimer < 0 then
			self.acTurnStage = 12;					
		end;

--==============================================================				
-- wait during U-turn before going forward
	elseif self.acTurnStage == 14 then
		allowedToDrive = false;						
		
		angle = 0;
		
		if self.turnTimer < 0 then
			self.acTurnStage = 15;					
		end;
		
--==============================================================				
-- go to the right distance before the U-turn
	elseif self.acTurnStage == 15 then

		angle = 0;
		
		local dist = self.acDimensions.uTurnDistance;
		if self.acParameters.noReverse then
			dist = self.acDimensions.uTurnDistance;
		end;
			
		if AutoSteeringEngine.getTurnDistance(self) > dist then
			self.acTurnStage = 16;					
			self.turnTimer   = self.acDeltaTimeoutRun;
		end;

--==============================================================				
-- wait during U-turn after going forward
	elseif self.acTurnStage == 16 then

		allowedToDrive = false;						
											
		angle = self.acDimensions.maxSteeringAngle;
		if self.turnTimer < 0 then
			if self.acParameters.leftAreaActive then
				AITractor.aiRotateLeft(self);
			else
				AITractor.aiRotateRight(self);
			end
			self.acTurnStage   = 17;					
			self.lastTurnAngle = math.deg(AutoSteeringEngine.getTurnAngle(self));					
		end;
		
--==============================================================				
-- The U-turn					
	elseif self.acTurnStage == 17 then

		local ref = -105;
				
		if not self.acParameters.noReverse and turnAngle <= ref then
			self.acTurn2Outside   = true;
			self.acTurnStage      = 18;
			self.turnTimer        = self.acDeltaTimeoutStop;
		elseif turnAngle <= -175 then 
			self.acTurnStage      = 19;				
			self.lastTurnAngle    = 0;
			self.turnTimer        = self.acDeltaTimeoutRun;
		elseif detected and turnAngle <= ref then
			self.acTurnStage      = 19;				
			self.lastTurnAngle    = 0;
			self.turnTimer        = self.acDeltaTimeoutRun;
		elseif fruitsDetected and not self.acParameters.noReverse and turnAngle <= -75 then
			self.acTurn2Outside   = true;
			self.acTurnStage      = 18;
			self.turnTimer        = self.acDeltaTimeoutStop;
		end
		
		angle = self.acDimensions.maxSteeringAngle;

--==============================================================				
-- going back
	elseif self.acTurnStage == 18 then

		moveForwards = false;					
		
		if      detected 
				and not fruitsDetected then
			self.acTurnStage    = 19;
			self.turnTimer      = self.acDeltaTimeoutRun;
		elseif math.abs(turnAngle) > 175 then
			self.acTurnStage    = 19;
			self.turnTimer      = self.acDeltaTimeoutRun;
			self.lastTurnAngle  = 0;
		end;

--==============================================================				
-- wait after U-turn
	elseif self.acTurnStage == 19 then
		--allowedToDrive = false;						
		
		angle = self.acLastSteeringAngle;
		self.acTurnStage = 29;					
		
--==============================================================				
	elseif self.acTurnStage == 29 then
		angle  = nil
		angle2 = AutoSteeringEngine.navigateToSavePoint( self, true, angleMax )
		if fruitsDetected or not AutoSteeringEngine.checkIsAnimPlaying( self ) then
			self.acTurnStage = -2;					
		end
		--	self.turnTimer   = self.acDeltaTimeoutStart;
		--end;

--==============================================================				
--==============================================================				
-- the new U-turn with reverse
	elseif self.acTurnStage == 20 then
		angle = 0;
		self.acTurnStage   = self.acTurnStage + 1;					
		self.turnTimer     = self.acDeltaTimeoutRun;

		if not self.acParameters.headland then
			self.setAIImplementsMoveDown(self,false);
		end
				
--==============================================================				
-- move far enough if tool is in front
	elseif self.acTurnStage == 21 then
		angle = 0;

		local x,z = AutoSteeringEngine.getTurnVector( self, true );
		if z > math.max( 0, self.acDimensions.toolDistance ) + 1 then
			if self.acParameters.headland then
				self.setAIImplementsMoveDown(self,false);
			end
			self.acTurnStage   = self.acTurnStage + 1;					
			self.turnTimer     = self.acDeltaTimeoutRun;
		end

--==============================================================				
-- turn 90°
	elseif self.acTurnStage == 22 then
		angle = self.acDimensions.maxSteeringAngle;
		
		local toolAngle = AutoSteeringEngine.getToolAngle( self );	
		if not self.acParameters.leftAreaActive then
			toolAngle = -toolAngle
		end
		
		angle = angle - math.max( 0, toolAngle - 1 );	
		
		if turnAngle < -135 or turnAngle + 0.4 * math.deg( toolAngle ) < -90 then
			self.acTurnStage   = self.acTurnStage + 1;					
			self.turnTimer     = self.acDeltaTimeoutRun;
		end

--==============================================================			
-- move forwards and reduce tool angle	
	elseif self.acTurnStage == 23 then
		angle  = nil;
		local toolAngle = AutoSteeringEngine.getToolAngle( self );		
		angle2 = math.min( math.max( -toolAngle, -self.acDimensions.maxSteeringAngle ), self.acDimensions.maxSteeringAngle );

		if math.abs(math.deg(toolAngle)) < 5 then
			if self.acParameters.leftAreaActive then
				AITractor.aiRotateLeft(self);
			else
				AITractor.aiRotateRight(self);
			end
			self.acTurnStage   = self.acTurnStage + 1;					
			self.turnTimer     = self.acDeltaTimeoutRun;
		end

--==============================================================				
-- wait		
	elseif self.acTurnStage == 24 then
		allowedToDrive = false;						
		moveForwards = false;					
		angle  = 0;
		
		if self.turnTimer < 0 then
			self.acTurnStage   = self.acTurnStage + 1;					
			self.turnTimer     = self.acDeltaTimeoutStop;
		end
		
--==============================================================				
-- move backwards (straight)		
	elseif self.acTurnStage == 25 then		
		moveForwards = false;					
		angle  = nil;
		local toolAngle = AutoSteeringEngine.getToolAngle( self );
		angle2 = math.min( math.max( toolAngle, -self.acDimensions.maxSteeringAngle ), self.acDimensions.maxSteeringAngle );
		
		local x,z = AutoSteeringEngine.getTurnVector( self, true );
		if self.acParameters.leftAreaActive then x = -x end
		x = x - 2
			
		if x > self.acDimensions.radius or z < 0 then
			self.acTurnStage   = self.acTurnStage + 1;					
			self.turnTimer     = self.acDeltaTimeoutRun;
		end
		
--==============================================================				
-- wait
	elseif self.acTurnStage == 26 then
		allowedToDrive = false;						
		angle = self.acDimensions.maxSteeringAngle;
		
		if self.turnTimer < 0 then
			self.acTurnStage   = self.acTurnStage + 1;					
			self.turnTimer     = self.acDeltaTimeoutStop;
		end

--==============================================================				
-- turn 90°
	elseif self.acTurnStage == 27 then
		local x,z = AutoSteeringEngine.getTurnVector( self, true );
--*********************************************************************************		
-- TODO
		local test= Utils.getNoNil( self.aseDistance , 0 )
--*********************************************************************************				
		
		if z >= test then
			if fruitsDetected then
				self.setAIImplementsMoveDown(self,true);
			end
		elseif not detected or not fruitsDetected then
			self.setAIImplementsMoveDown(self,false);
		end
		
		if fruitsDetected and detected and z < test then
			self.setAIImplementsMoveDown(self,true);
			self.acTurnStage   = -22 --self.acTurnStage + 1;					
			self.turnTimer     = self.acDeltaTimeoutNoTurn;
		elseif not detected then					
			angle  = nil
			angle2 = AutoSteeringEngine.navigateToSavePoint( self, true, angleMax )
		end
		
--==============================================================				
-- wait after U-turn
	elseif self.acTurnStage == 28 then
		allowedToDrive   = false;						
		angle            = 0;		
		self.acTurnStage = -22 -- -2;					
	
--==============================================================				
--==============================================================				
-- 90° turn to inside with reverse
	elseif self.acTurnStage == 30 then

		self.setAIImplementsMoveDown(self,false);
		self.acTurnStage   = self.acTurnStage + 1;
		self.turnTimer     = self.acDeltaTimeoutWait;
		--self.waitForTurnTime = self.time + self.turnTimer;

--==============================================================				
-- wait
	elseif self.acTurnStage == 31 then
		allowedToDrive = false;				
		moveForwards = false;					
		angle = 0
		
		if self.turnTimer < 0 then
			self.acTurnStage   = self.acTurnStage + 1;					
		end

--==============================================================				
-- move backwards (straight)		
	elseif self.acTurnStage == 32 then		
		moveForwards = false;					
		angle  = nil;
		local toolAngle = AutoSteeringEngine.getToolAngle( self );
		angle2 = math.min( math.max( toolAngle, -self.acDimensions.maxSteeringAngle ), self.acDimensions.maxSteeringAngle );

		local _,z = AutoSteeringEngine.getTurnVector( self );
		if z < 0.7 * self.acDimensions.radius then
			self.acTurnStage   = self.acTurnStage + 1;					
			self.turnTimer     = self.acDeltaTimeoutRun;
		end

--==============================================================				
-- turn 45°
	elseif self.acTurnStage == 33 then
		angle = -self.acDimensions.maxSteeringAngle;
		
		local toolAngle = AutoSteeringEngine.getToolAngle( self );	
		if self.acParameters.leftAreaActive then
			toolAngle = -toolAngle
		end
		
		angle = angle + math.max( 0, toolAngle - 0.7 );	
		
		if turnAngle - 0.5 * math.deg( toolAngle ) > 42 then
			self.acTurnStage   = self.acTurnStage + 1;					
			self.turnTimer     = self.acDeltaTimeoutRun;
		end

--==============================================================			
-- move forwards and reduce tool angle	
	elseif self.acTurnStage == 34 then
		angle  = nil;
		local toolAngle = AutoSteeringEngine.getToolAngle( self );		
		angle2 = math.min( math.max( -toolAngle, -self.acDimensions.maxSteeringAngle ), self.acDimensions.maxSteeringAngle );

		if math.abs(math.deg(toolAngle)) < 5 then
			self.acTurnStage   = self.acTurnStage + 1;					
			self.turnTimer     = self.acDeltaTimeoutRun;
		end

--==============================================================				
-- wait		
	elseif self.acTurnStage == 35 then
		allowedToDrive = false;						
		moveForwards = false;					
		angle  = 0;

		if self.turnTimer < 0 then
			self.acTurnStage   = self.acTurnStage + 1;					
			self.turnTimer     = self.acDeltaTimeoutStop;
		end
		
--==============================================================				
-- move backwards (straight)		
	elseif self.acTurnStage == 36 then		
		moveForwards = false;					
		angle  = nil;
		local toolAngle = AutoSteeringEngine.getToolAngle( self );
		angle2 = math.min( math.max( toolAngle, -self.acDimensions.maxSteeringAngle ), self.acDimensions.maxSteeringAngle );
		
		local _,z = AutoSteeringEngine.getTurnVector( self );
		
		if z < 0 or ( z < 0.5 * self.acDimensions.distance and detected ) then				
			self.acTurnStage   = self.acTurnStage + 1;					
			self.turnTimer     = self.acDeltaTimeoutRun;
		end
		
--==============================================================				
-- wait
	elseif self.acTurnStage == 37 then
		allowedToDrive = false;						
		angle = -self.acDimensions.maxSteeringAngle;
		
		if self.turnTimer < 0 then
			self.acTurnStage   = self.acTurnStage + 1;					
			self.turnTimer     = self.acDeltaTimeoutStop;
		end

--==============================================================				
-- turn 45°
	elseif self.acTurnStage == 38 then
		local x = AutoSteeringEngine.getTurnVector( self );
		if self.acParameters.leftAreaActive then x = -x end
		
		if turnAngle < 70 then
			angle = -self.acDimensions.maxSteeringAngle;
		elseif fruitsDetected or detected or math.abs( turnAngle ) > 90 or x < 0 then
			self.acTurnStage = -1;					
			self.turnTimer   = self.acDeltaTimeoutStart;
		else
			angle = 0
		end
		
--==============================================================				
-- wait after 90° turn
	elseif self.acTurnStage == 39 then
		allowedToDrive = false;						
		
		angle = 0;
		
		--if self.turnTimer < 0 then
			self.acTurnStage = -1;					
		--	self.turnTimer   = self.acDeltaTimeoutStart;
		--end;

--==============================================================				
--==============================================================				
-- 180° turn with 90° backwards
	elseif self.acTurnStage == 40 then
		angle = 0;
		self.acTurnStage   = self.acTurnStage + 1;					
		self.turnTimer     = self.acDeltaTimeoutRun;

		if not self.acParameters.headland then
			self.setAIImplementsMoveDown(self,false);
		end
		
--==============================================================				
-- move far enough if tool is in front
	elseif self.acTurnStage == 41 then
		angle = 0;

		local x,z = AutoSteeringEngine.getTurnVector( self, true );
		if z > math.max( 0, self.acDimensions.toolDistance ) + 1 then
			if self.acParameters.headland then
				self.setAIImplementsMoveDown(self,false);
			end
			self.acTurnStage   = self.acTurnStage + 1;					
			self.turnTimer     = self.acDeltaTimeoutRun;
		end

--==============================================================				
-- wait
	elseif self.acTurnStage == 42 then
		allowedToDrive = false;				
		moveForwards = false;					
		angle = 0
		
		if self.turnTimer < 0 then
			self.acTurnStage   = self.acTurnStage + 1;					
		end

--==============================================================				
-- turn 45°
	elseif self.acTurnStage == 43 then
		angle = -self.acDimensions.maxSteeringAngle;
		
		local toolAngle = AutoSteeringEngine.getToolAngle( self );	
		if self.acParameters.leftAreaActive then
			toolAngle = -toolAngle
		end
		
		angle = angle + math.max( 0, toolAngle - 0.7 );	
		
		if turnAngle > 42 then
			if self.acParameters.leftAreaActive then
				AITractor.aiRotateLeft(self);
			else
				AITractor.aiRotateRight(self);
			end
			self.acTurnStage   = self.acTurnStage + 1;					
			self.turnTimer     = self.acDeltaTimeoutNoTurn;
		end
--==============================================================				
-- wait
	elseif self.acTurnStage == 44 then
		allowedToDrive = false;						
		angle = self.acDimensions.maxSteeringAngle;
		
		if self.turnTimer < 0 then
			self.acTurnStage   = self.acTurnStage + 1;					
			self.turnTimer     = self.acDeltaTimeoutStop;
		end

--==============================================================				
-- move backwards (90°)	I	
	elseif self.acTurnStage == 45 then		
		moveForwards = false;					
		angle = self.acDimensions.maxSteeringAngle;
		
		if turnAngle > 87 then
			self.acTurnStage   = self.acTurnStage + 1;					
			self.turnTimer     = self.acDeltaTimeoutRun;
		end
--==============================================================				
-- move backwards (0°) II
	elseif self.acTurnStage == 46 then		
		moveForwards = false;					
		angle = math.rad(1);
		local x,z = AutoSteeringEngine.getTurnVector( self, true );
		if not self.acParameters.leftAreaActive then x = -x end
	
		if x > 0 then
			self.acTurnStage   = self.acTurnStage + 1;					
			self.turnTimer     = self.acDeltaTimeoutRun;
		end
--==============================================================				
-- move backwards (90°) III
	elseif self.acTurnStage == 47 then		
		moveForwards = false;					
		angle = self.acDimensions.maxSteeringAngle;
		
		if turnAngle > 115 then
			self.acTurnStage   = self.acTurnStage + 1;					
			self.turnTimer     = self.acDeltaTimeoutRun;
		end
--==============================================================				
-- wait
	elseif self.acTurnStage == 48 then
		allowedToDrive = false;						
		angle = -self.acDimensions.maxSteeringAngle;
		
		if self.turnTimer < 0 then
			self.acTurnStage   = self.acTurnStage + 1;					
			self.turnTimer     = self.acDeltaTimeoutStop;
		end

--==============================================================				
-- turn 90° II
	elseif self.acTurnStage == 49 then
		local x,z = AutoSteeringEngine.getTurnVector( self, true );
--*********************************************************************************		
-- TODO
		local test= Utils.getNoNil( self.aseDistance , 0 )
--*********************************************************************************				
		
		if z >= test then
			if fruitsDetected then
				self.setAIImplementsMoveDown(self,true);
			end
		elseif not detected or not fruitsDetected then
			self.setAIImplementsMoveDown(self,false);
		end
		
		if fruitsDetected and detected and z < test then
			self.setAIImplementsMoveDown(self,true);
			self.acTurnStage   = -22 --self.acTurnStage + 1;					
			self.turnTimer     = self.acDeltaTimeoutNoTurn;
		elseif not detected then					
			angle  = nil
			angle2 = AutoSteeringEngine.navigateToSavePoint( self, true, angleMax )
		end

		
--==============================================================				
--==============================================================				
-- 180° turn with 90° backwards
	elseif self.acTurnStage == 50 then
		allowedToDrive = false;				
		moveForwards = false;					
		angle = 0
		
		--if self.turnTimer < 0 then
			self.setAIImplementsMoveDown(self,false);
			self.acTurnStage   = self.acTurnStage + 1;					
		--end

--==============================================================				
-- turn 45°
	elseif self.acTurnStage == 51 then
		angle = -self.acDimensions.maxSteeringAngle;
		moveForwards = false;					
		
		if turnAngle < -42 then
			if self.acParameters.leftAreaActive then
				AITractor.aiRotateLeft(self);
			else
				AITractor.aiRotateRight(self);
			end
			self.acTurnStage   = self.acTurnStage + 1;					
			self.turnTimer     = self.acDeltaTimeoutNoTurn;
		end
--==============================================================				
-- wait
	elseif self.acTurnStage == 52 then
		allowedToDrive = false;						
		angle = self.acDimensions.maxSteeringAngle;
		
		if self.turnTimer < 0 then
			self.acTurnStage   = self.acTurnStage + 1;					
			self.turnTimer     = self.acDeltaTimeoutStop;
		end

--==============================================================				
-- move backwards (90°)	I	
	elseif self.acTurnStage == 53 then		
		angle = self.acDimensions.maxSteeringAngle;
		
		if turnAngle < -80 then
			self.acTurnStage   = self.acTurnStage + 1;					
			self.turnTimer     = self.acDeltaTimeoutRun;
		end
--==============================================================				
-- move backwards (0°) II
	elseif self.acTurnStage == 54 then		
		angle = math.rad(-90-turnAngle);
		local x,z = AutoSteeringEngine.getTurnVector( self, true );
		if not self.acParameters.leftAreaActive then x = -x end
	
		if x > 0 then
			self.acTurnStage   = self.acTurnStage + 1;					
			self.turnTimer     = self.acDeltaTimeoutRun;
		end
--==============================================================				
-- move backwards (90°) III
	elseif self.acTurnStage == 55 then		
		angle = self.acDimensions.maxSteeringAngle;
		
		if turnAngle < -115 then
			self.acTurnStage   = self.acTurnStage + 1;					
			self.turnTimer     = self.acDeltaTimeoutRun;
		end
--==============================================================				
-- wait
	elseif self.acTurnStage == 56 then
		allowedToDrive = false;						
		moveForwards = false;					
		angle = -self.acDimensions.maxSteeringAngle;
		
		if self.turnTimer < 0 then
			self.acTurnStage   = self.acTurnStage + 1;					
			self.turnTimer     = self.acDeltaTimeoutStop;
		end

--==============================================================				
-- move backwards (90°)	I	
	elseif self.acTurnStage == 57 then		
		angle = -self.acDimensions.maxSteeringAngle;
		moveForwards = false;					

		
		if turnAngle > 0 or turnAngle < -175 then
			AutoSteeringEngine.setChainStraight( self );		
			detected, angle2, border = AutoSteeringEngine.processChain( self )
			if detected then
				angle  = nil
				angle2 = -angle2
			else
				angle  = 0
				angle2 = nil
			end
			
			local x,z = AutoSteeringEngine.getTurnVector( self, true );
			if not self.acParameters.leftAreaActive then x = -x end
			
			if z > self.acDimensions.toolDistance then	
				if detected or z > self.acDimensions.toolDistance + 3 then	
					self.acTurnStage   = self.acTurnStage + 1;					
					self.turnTimer     = self.acDeltaTimeoutRun;
				end
			end
		end

--==============================================================				
-- wait
	elseif self.acTurnStage == 58 then
		allowedToDrive = false;						
		angle = 0
		
		if self.turnTimer < 0 then
			self.acTurnStage   = self.acTurnStage + 1;					
			self.turnTimer     = self.acDeltaTimeoutStop;
		end

		--==============================================================				
-- turn 90° II
	elseif self.acTurnStage == 59 then
		local x,z = AutoSteeringEngine.getTurnVector( self, true );
--*********************************************************************************		
-- TODO
		local test= Utils.getNoNil( self.aseDistance , 0 )
--*********************************************************************************				
		
		if z >= test then
			if fruitsDetected then
				self.setAIImplementsMoveDown(self,true);
			end
		elseif not detected or not fruitsDetected then
			self.setAIImplementsMoveDown(self,false);
		end
		
		if fruitsDetected and detected and z < test then
			self.setAIImplementsMoveDown(self,true);
			self.acTurnStage   = -22 --self.acTurnStage + 1;					
			self.turnTimer     = self.acDeltaTimeoutNoTurn;
		elseif not detected then					
			angle  = nil
			angle2 = AutoSteeringEngine.navigateToSavePoint( self, true, angleMax )
		end

		
--==============================================================				
--==============================================================				
-- searching...
	elseif ( -3 <= self.acTurnStage and self.acTurnStage < 0 )
			or (-13 <= self.acTurnStage and self.acTurnStage < -10 )then
		moveForwards     = true;

		if fruitsDetected and self.acTurnStage > -3 then
			self.setAIImplementsMoveDown(self,true);
			self.acTurnStage = self.acTurnStage -20;
			self.turnTimer   = self.acDeltaTimeoutNoTurn;
		
		elseif fruitsDetected and detected then
			if not ( self.acTurnStage == -11 and self.acParameters.upNDown ) then
				AutoSteeringEngine.clearTrace( self );
				AutoSteeringEngine.saveDirection( self, false );
			end
			self.acTurnStage    = 0;
			self.acTurn2Outside = false;
			self.turnTimer      = self.acDeltaTimeoutNoTurn;
			self.aiRescueTimer  = self.acDeltaTimeoutStop;
		end;
		
--==============================================================				
	elseif -23 <= self.acTurnStage and self.acTurnStage < -20 then
		allowedToDrive = false;
		
		--if self.turnTimer < 0 then
			self.acTurnStage = self.acTurnStage + 10;					
		--end;
		
--==============================================================				
-- threshing...					
	elseif self.acTurnStage == 0 then
		moveForwards     = true;
		
		local traceLength = AutoSteeringEngine.getTraceLength(self);
		
		local doTurn = false;
		local uTurn  = false;
		
		if  detected then
			doTurn = false
		elseif  fruitsDetected 
				and not self.acTurn2Outside then
			doTurn = false
		elseif  self.turnTimer < 0 then
			doTurn = true
			if     self.acTurn2Outside 
					or not self.acParameters.upNDown
					or traceLength < 10 then		
				uTurn = false
			else
				uTurn = true
			end
		end
		
		if doTurn then
			if     uTurn               then
		-- the U turn
				--invert turn angle because we will swap left/right in about 10 lines
				
				turnAngle = -turnAngle;
				if self.acParameters.noReverse then
					if self.acParameters.noReverse or self.acDimensions.uTurnAngle > 1 then
						self.acTurn2Outside = turnAngle < self.acDimensions.uTurnAngle;
					else
						self.acTurn2Outside = false; --math.deg(turnAngle) < -3;					
					end;
					self.acTurnStage = 11; 					
				elseif noReverseIndex <= 0 and not self.isRealistic then
					self.acTurnStage = 50;
				elseif noReverseIndex <= 0 and self.acDimensions.distance < self.acDimensions.radius + 1.5 then
					self.acTurnStage = 40;
				else
					self.acTurnStage = 20;
				end
				self.turnTimer = self.acDeltaTimeoutWait;
				self.waitForTurnTime = self.time + self.turnTimer;
				self.acParameters.leftAreaActive  = not self.acParameters.leftAreaActive;
				self.acParameters.rightAreaActive = not self.acParameters.rightAreaActive;
				AutoTractor.sendParameters(self);
				AutoSteeringEngine.setChainStraight( self );	
			elseif self.acTurn2Outside then
		-- turn to outside because we are in the middle of the field
				self.acTurnStage = 1;
				self.turnTimer = self.acDeltaTimeoutWait;
			elseif self.acParameters.noReverse then
		-- 90° turn w/o reverse
				self.aiRescueTimer  = 3 * self.acDeltaTimeoutStop;
				self.acTurnStage = 5;
				self.turnTimer = self.acDeltaTimeoutWait;
				self.waitForTurnTime = self.time + self.turnTimer;
				--if not self.acParameters.upNDown
				--		or traceLength < 10 then
					self.acTurn2Outside = false
				--else
				--	self.acTurn2Outside = true
				--	self.acParameters.leftAreaActive  = not self.acParameters.leftAreaActive;
				--	self.acParameters.rightAreaActive = not self.acParameters.rightAreaActive;
				--	AutoTractor.sendParameters(self);
				--end
			elseif noReverseIndex <= 0 then --and ( traceLength < 6 or self.acDimensions.distance < 2 ) then
		-- 90° turn with reverse
				self.acTurnStage = 1;
				self.turnTimer = self.acDeltaTimeoutWait;
			else
		-- 90° turn with reverse
				self.acTurnStage = 30;
				self.turnTimer = self.acDeltaTimeoutWait;
			end
		elseif detected then --and fruitsDetected then
			AutoSteeringEngine.saveDirection( self, true );
			self.turnTimer   	  = math.max(self.turnTimer,self.acDeltaTimeoutRun);
			self.aiRescueTimer  = self.acDeltaTimeoutStop;
		elseif  fruitsDetected 
				and not self.acTurn2Outside then
			AutoSteeringEngine.saveDirection( self, true );
		end
		
--==============================================================				
--==============================================================				
-- error!!!
	else
		allowedToDrive = false;						
		self.mogliInfoText = string.format(Mogli.getText("AC_COMBINE_ERROR")..": %i",self.acTurnStage);
		print(self.mogliInfoText);
		self:stopAITractor();
		return;
	end;                

--==============================================================				
--==============================================================				
	if math.abs( self.axisSide ) > 0.3 then --if AutoTractor.acDevFeatures and math.abs( self.axisSide ) > 0.3 then 
		detected = false
		border   = 0
		angle    = nil
		angle2   = - self.axisSide * self.acDimensions.maxSteeringAngle
		self.turnTimer = self.turnTimer + dt;
		self.turnTimer = self.turnTimer + dt;
		if self.acTurnStage <= 0 then
			self.aiRescueTimer = self.aiRescueTimer + dt;
		end			
	end			
--==============================================================				
--==============================================================				

	if     self.acTurnStage <= 0 and detected then
		AutoTractor.setStatus( self, 1 )
	elseif self.acTurnStage <= 0 then
		AutoTractor.setStatus( self, 2 )
	else
		AutoTractor.setStatus( self, 0 )
	end
	
	local acceleration   = 0;					
	local slowAngleLimit = self.acDimensions.maxLookingAngle;
	
	if self.isMotorStarted and speedLevel ~= 0 and self.fuelFillLevel > 0 then
		acceleration = 1.0;
	end;

	if speedLevel == 0 then
		allowedToDrive = false
		speedLevel     = 1
	elseif self.acTurnStage > 0 
			or ( not detected and self.acTurnStage >= -2 ) then
		speedLevel = 4
		slowAngleLimit = self.acDimensions.maxSteeringAngle;
	end;
	
	if self.acTurnStage > 0 then
		detected = false;
	end
	
	if angle == nil then
		if angle2 == nil then
			angle = 0;
		else
			angle = angle2;
		end
	elseif not self.acParameters.leftAreaActive then
		angle = -angle;		
	end
	
	if AutoTractor.acDevFeatures then
		self.mogliInfoText = self.mogliInfoText .. string.format(" a: %3i ",math.deg(angle))..tostring(detected)
	end

	local aiSteeringSpeed = self.aiSteeringSpeed;
	if detected and 
			(  math.abs( angle ) > 0.5 * angleMax
			or math.abs( angle - AutoSteeringEngine.currentSteeringAngle( self ) )  > 0.25 * angleMax
			or self.acTurnStage == -3
			or self.acTurnStage == -13
			or -3 <= self.acTurnStage and self.acTurnStage < 0 ) then
		detected = false
	end
	
	if     self.isRealistic then
		aiSteeringSpeed = 3 * aiSteeringSpeed
	elseif speedLevel == 4 then
		aiSteeringSpeed = 0.5 * aiSteeringSpeed
	elseif speedLevel == 2 then
		aiSteeringSpeed = 3 * aiSteeringSpeed
	elseif speedLevel == 3 then
		aiSteeringSpeed = 6 * aiSteeringSpeed
	end
	
	if self.isRealistic then
		if      self.motor                   ~= nil
				and self.motor.realSpeedLevelsAI ~= nil
				and self.acRealSpeedLevelsAI1    ~= nil then
			self.motor.realSpeedLevelsAI[1] = self.acRealSpeedLevelsAI1
		end		
		
	  if AIVehicleUtil.mrDriveInDirection == nil then
			if self.acTurnStage==0 then
				self.turnStage = 0
			else
				self.turnStage = 2
				speedLevel = 1
			end
			if speedLevel == 4 then
				speedLevel = 1
			end
		elseif self.acTurnStage==0 then -- when the AI is working, we set the speedlevel adequately to the implements
			self.turnStage = 0
			if not detected then
				speedLevel = 3; -- updated in OverrideAITractor.lua (updateToolsInfo)
			else
				speedLevel = 1; -- updated in OverrideAITractor.lua (updateToolsInfo)		
				if      2 <= self.speed2Level and self.speed2Level <= 4 
						and self.acRealSpeedLevelsAI1  ~= nil
						and self.motor.realSpeedLevels ~= nil
						and self.motor.realSpeedLevels[self.speed2Level-1] ~= nil then
					self.motor.realSpeedLevelsAI[1] = self.motor.realSpeedLevels[self.speed2Level-1]
				end
			end;
		else
			self.turnStage = 2
			speedLevel     = 3 --2; -- set in RealisticMotorized.lua (load function) => ai maneuver speed		
		end;
		
		if math.abs( self.realGroundSpeed ) > 0.5 and not allowedToDrive then
			angle = 0
		end
		
	elseif not detected and 0 < speedLevel and speedLevel < 4 then
		speedLevel = 4
	elseif speedLevel ~= 2 and speedLevel ~= 3 and speedLevel ~= 4 then
		speedLevel = 1
	end
	
	angle = math.min( math.max( angle, -self.acDimensions.maxSteeringAngle ), self.acDimensions.maxSteeringAngle )
	
	if AutoTractor.acDevFeatures then
		self.mogliInfoText = self.mogliInfoText .." "..tostring(allowedToDrive).." "..tostring(moveForwards).." "..tostring(speedLevel)
	end

	AutoSteeringEngine.steer( self, dt, angle, aiSteeringSpeed, detected );
	AutoSteeringEngine.drive( self, dt, acceleration, 0.75, angle, slowAngleLimit, allowedToDrive, moveForwards, speedLevel, 0.75 );
	
  local colDirX = math.sin( angle );
	local colDirZ = math.cos( angle );

	if not moveForwards then
		colDirZ = -colDirZ;
	end

--	if self.isRealistic then
--		local lx, lz
--		lx = colDirX
--		lz = colDirZ
--			
--		if AIVehicleUtil.mrDriveInDirection == nil then
----function AIVehicleUtil:driveInDirection(dt, steeringAngleLimit, acceleration, slowAcceleration, slowAngleLimit, allowedToDrive, moveForwards, lx, lz, speedLevel, slowMaxRpmFactor)
--			if not moveForwards then
--				lz = -lz
--			end
--			AIVehicleUtil.driveInDirection(self, dt, self.acDimensions.maxSteeringAngle, 0.5, 0.5, angleMax, allowedToDrive, moveForwards, lx, lz, speedLevel, 0.9);
--		else
--			if not moveForwards then
--				lx = -lx
--			end
-- --*************************************************** DURAL ************************************************************************************				
--			if allowedToDrive then
--				AIVehicleUtil.mrDriveInDirection(self, dt, 1, true, moveForwards, lx, lz, speedLevel, not detected);
--			else			
--				AIVehicleUtil.mrDriveInDirection(self, dt, 0, false, moveForwards);				
--			end;
----*************************************************** END DURAL ************************************************************************************
--		end
--	else
--		AutoSteeringEngine.steer( self, dt, angle, aiSteeringSpeed, detected );
--		AutoSteeringEngine.drive( self, dt, acceleration, 0.75, angle, slowAngleLimit, allowedToDrive, moveForwards, speedLevel, 0.75 );
--	end
	
	for triggerId, _ in pairs(self.numCollidingVehicles) do
		AIVehicleUtil.setCollisionDirection(self.aiTractorDirectionNode, triggerId, colDirX, colDirZ)
	end

end
